﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Drawing;
using System.Management;
using System.Text.RegularExpressions;
using System.Windows.Forms;


namespace RobvanderWoude
{
	class PrinterSelectBox
	{
		public static string progver = "2.01";

		public static ComboBox combobox;
		public static Label statusField;
		public static Label typeField;
		public static Label whereField;
		public static Label commentField;
		public static List<string[]> printerList;

		[STAThread]
		static int Main( string[] args )
		{
			try
			{
				// Inspired by code by Gorkem Gencay on StackOverflow.com:
				// http://stackoverflow.com/questions/97097/what-is-the-c-sharp-version-of-vb-nets-inputdialog#17546909

				string selected;
				string title = "Select Printer";
				string preselected = String.Empty;
				SortedList printerNames;
				int width = 400;
				int height = 220;

				if ( args.Length > 3 )
				{
					return WriteError( );
				}
				foreach ( string arg in args )
				{
					if ( arg == "/?" )
					{
						return WriteError( );
					}
				}

				QueryPrinters( );
				printerNames = new SortedList( );
				foreach ( string[] prn in printerList )
				{
					printerNames.Add( prn[0], prn[5] );
					// Make the default printer the initial preselected printer in the dropdown list
					if ( prn[5] == "1" )
					{
						preselected = prn[0];
					}
				}

				if ( args.Length > 0 )
				{
					if ( !String.IsNullOrWhiteSpace( args[0] ) )
					{
						title = args[0];
					}
					if ( args.Length > 1 )
					{
						if ( !String.IsNullOrWhiteSpace( args[1] ) )
						{
							// Make the specified printer the selected one in the dropdown list
							if ( printerNames.ContainsKey( args[1] ) )
							{
								preselected = args[1];
							}
							else
							{
								string pattern = args[1];
								bool match = false;
								foreach ( string printer in printerNames.Keys )
								{
									if ( Regex.IsMatch( printer, pattern, RegexOptions.IgnoreCase ) )
									{
										match = true;
										preselected = printer;
										break; // Search no further after first match
									}
								}
								if ( !match )
								{
									return WriteError( String.Format( "No printer name found matching \"{0}\"", preselected ) );
								}
							}
						}
						if ( args.Length > 2 )
						{
							try
							{
								width = Convert.ToInt32( args[2] );
								if ( width < 400 || width > Screen.PrimaryScreen.Bounds.Width )
								{
									return WriteError( String.Format( "Invalid screen width: \"{0}\".\n\tWidth must be an integer between 400 and {1} (screen width).", args[2], Screen.PrimaryScreen.Bounds.Width ) );
								}
							}
							catch ( FormatException e )
							{
								return WriteError( String.Format( "Invalid width: \"{0}\"\n\t{1}.", args[2], e.Message ) );
							}
						}
					}
				}

				// Create the dialog box
				Form printerSelect = new Form( );
				Size size = new Size( width, height );
				printerSelect.FormBorderStyle = FormBorderStyle.FixedDialog;
				printerSelect.MaximizeBox = false;
				printerSelect.MinimizeBox = false;
				printerSelect.StartPosition = FormStartPosition.CenterParent;
				printerSelect.ClientSize = size;
				printerSelect.Text = title;

				// Label "Name:" for printer name
				Label labelName = new Label( );
				labelName.Size = new Size( 70, 20 );
				labelName.Location = new Point( 10, 27 );
				labelName.Text = "Name:";
				printerSelect.Controls.Add( labelName );

				// Dropdown list to contain available printer names
				combobox = new ComboBox( );
				combobox.Size = new Size( size.Width - 100, 25 );
				combobox.Location = new Point( 90, 25 );
				combobox.AutoCompleteMode = AutoCompleteMode.Append;
				combobox.AutoCompleteSource = AutoCompleteSource.ListItems;
				combobox.DropDownStyle = ComboBoxStyle.DropDownList;
				combobox.SelectedIndexChanged += new EventHandler( combobox_SelectedIndexChanged );
				printerSelect.Controls.Add( combobox );

				// Label "Status:" for selected printer's current status
				Label labelStatus = new Label( );
				labelStatus.Size = new Size( 70, 20 );
				labelStatus.Location = new Point( 10, 57 );
				labelStatus.Text = "Status:";
				printerSelect.Controls.Add( labelStatus );

				// Label to contain selected printer's current status
				statusField = new Label( );
				statusField.Size = new Size( size.Width - 10 - 80, 20 );
				statusField.Location = new Point( 90, 57 );
				statusField.Text = "Status Value";
				printerSelect.Controls.Add( statusField );

				// Label "Type:" for selected printer's type description
				Label labelType = new Label( );
				labelType.Size = new Size( 70, 20 );
				labelType.Location = new Point( 10, 82 );
				labelType.Text = "Type:";
				printerSelect.Controls.Add( labelType );

				// Label to contain selected printer's type description
				typeField = new Label( );
				typeField.Size = new Size( size.Width - 10 - 80, 20 );
				typeField.Location = new Point( 90, 82 );
				typeField.Text = "Type Value";
				printerSelect.Controls.Add( typeField );

				// Label "Where:" for selected printer's location or printer port
				Label labelWhere = new Label( );
				labelWhere.Size = new Size( 70, 20 );
				labelWhere.Location = new Point( 10, 107 );
				labelWhere.Text = "Where:";
				printerSelect.Controls.Add( labelWhere );

				// Label to contain selected printer's location or printer port
				whereField = new Label( );
				whereField.Size = new Size( size.Width - 10 - 80, 20 );
				whereField.Location = new Point( 90, 107 );
				whereField.Text = "Where Value";
				printerSelect.Controls.Add( whereField );

				// Label "Comment:" for selected printer's optional comment line
				Label labelComment = new Label( );
				labelComment.Size = new Size( 70, 20 );
				labelComment.Location = new Point( 10, 132 );
				labelComment.Text = "Comment:";
				printerSelect.Controls.Add( labelComment );

				// Label to contain selected printer's optional comment line
				commentField = new Label( );
				commentField.Size = new Size( size.Width - 10 - 80, 20 );
				commentField.Location = new Point( 90, 132 );
				commentField.Text = "Comment Value";
				printerSelect.Controls.Add( commentField );

				Button okButton = new Button( );
				okButton.DialogResult = DialogResult.OK;
				okButton.Name = "okButton";
				okButton.Size = new Size( 80, 25 );
				okButton.Text = "&OK";
				okButton.Location = new Point( size.Width / 2 - 10 - 80, 175 );
				printerSelect.Controls.Add( okButton );

				Button cancelButton = new Button( );
				cancelButton.DialogResult = DialogResult.Cancel;
				cancelButton.Name = "cancelButton";
				cancelButton.Size = new Size( 80, 25 );
				cancelButton.Text = "&Cancel";
				cancelButton.Location = new Point( size.Width / 2 + 10, 175 );
				printerSelect.Controls.Add( cancelButton );

				printerSelect.AcceptButton = okButton;  // OK on Enter
				printerSelect.CancelButton = cancelButton; // Cancel on Esc
				printerSelect.Activate( );

				// Populate the dropdown list with the printers from the sorted list
				int index = 0;
				foreach ( string prn in printerNames.Keys )
				{
					combobox.Items.Add( prn );
					// Select the preselected printer if specified, or the default printer otherwise
					if ( prn.ToLower( ) == preselected.ToLower( ) )
					{
						combobox.SelectedIndex = index;
						foreach ( string[] printer in printerList )
						{
							if ( printer[0].ToLower( ) == preselected.ToLower( ) )
							{
								statusField.Text = printer[1];
								typeField.Text = printer[2];
								whereField.Text = printer[3];
								commentField.Text = printer[4];
							}
						}
					}
					index += 1;
				}

				DialogResult result = printerSelect.ShowDialog( );
				if ( result == DialogResult.OK )
				{
					selected = combobox.SelectedItem.ToString( );
					Console.WriteLine( selected );
					return 0;
				}
				else
				{
					return 2;
				}
			}
			catch ( Exception e )
			{
				return WriteError( e.Message );
			}
		}


		public static void combobox_SelectedIndexChanged( object sender, EventArgs e )
		{
			string printername = combobox.Text;
			foreach ( string[] prn in printerList )
			{
				if ( prn[0] == printername )
				{
					statusField.Text = prn[1];
					typeField.Text = prn[2];
					whereField.Text = prn[3];
					commentField.Text = prn[4];
				}
			}
		}

		public static void QueryPrinters( )
		{
			printerList = new List<string[]>( );

			string query = "SELECT * FROM Win32_Printer";
			ManagementObjectSearcher searcher = new ManagementObjectSearcher( "root\\CIMV2", query );
			foreach ( ManagementObject queryObj in searcher.Get( ) )
			{
				string printer = (string) queryObj["DeviceID"];
				UInt32 printerstatus = Convert.ToUInt32( queryObj["ExtendedPrinterStatus"] );
				string printertype = (string) queryObj["DriverName"];
				string printerlocation = (string) queryObj["Location"];
				string printerport = (string) queryObj["PortName"];
				string printercomment = (string) queryObj["Comment"];
				bool isdefault = (bool) queryObj["Default"];
				string[] printerProperties = new string[6];
				printerProperties[0] = printer;
				printerProperties[1] = ( (WMIPrinterStatus) printerstatus ).ToString( );
				printerProperties[2] = printertype;
				if ( String.IsNullOrWhiteSpace( printerlocation ) )
				{
					printerProperties[3] = printerport;
				}
				else
				{
					printerProperties[3] = printerlocation;
				}
				printerProperties[4] = printercomment;
				printerProperties[5] = ( isdefault ? "1" : "0" );
				printerList.Add( printerProperties );
			}
		}

		public static void UpdateFields( )
		{
			string printername = combobox.SelectedValue.ToString( );
			foreach ( string[] prn in printerList )
			{
				if ( prn[0] == printername )
				{
					statusField.Text = prn[1];
					typeField.Text = prn[2];
					whereField.Text = prn[3];
					commentField.Text = prn[4];
				}
			}
		}

		#region Error handling

		public static int WriteError( string errorMessage = null )
		{
			/*
			PrinterSelectBox,  Version 2.00
			Batch tool to present a Printer Select dialog and return the selected printer

			Usage:    PRINTERSELECTBOX  [ "title"  [ "selected"  [ width ] ] ]

			Where:    "title"     is the optional caption in the title bar
			          "selected"  is the optional selected printer, either its full name or
			                      a regular expression (default: the default printer name)
			          width       is the optional width of the input box
			                      (default: 200; minimum: 200; maximum: screen width)

			Notes:    The name of the selected printer is written to Standard Out if "OK"
			          is clicked, otherwise an empty string is returned.
			          If "selected" is specified, the program will try an exact (literal)
			          match first; if no match is found, "selected" will be interpreted as
			          a regular expression, and the first match in the sorted printer list
			          will be used.
			          Return code 0 for \"OK\", 1 for (command line) errors, 2 for \"Cancel\".

			Credits:  Based on code by Gorkem Gencay on StackOverflow:
			          http://stackoverflow.com/questions/97097
			          /what-is-the-c-sharp-version-of-vb-nets-inputdialog#17546909

			Written by Rob van der Woude
			http://www.robvanderwoude.com
			*/

			if ( string.IsNullOrEmpty( errorMessage ) == false )
			{
				Console.Error.WriteLine( );
				Console.ForegroundColor = ConsoleColor.Red;
				Console.Error.Write( "ERROR:\t" );
				Console.ForegroundColor = ConsoleColor.White;
				Console.Error.WriteLine( errorMessage );
				Console.ResetColor( );
			}

			Console.Error.WriteLine( );
			Console.Error.WriteLine( "PrinterSelectBox,  Version {0}", progver );
			Console.Error.WriteLine( "Batch tool to present a Printer Select dialog and return the selected printer" );
			Console.Error.WriteLine( );
			Console.Error.Write( "Usage:    " );
			Console.ForegroundColor = ConsoleColor.White;
			Console.Error.WriteLine( "PRINTERSELECTBOX  [ \"title\"  [ \"selected\"  [ width ] ] ]" );
			Console.ResetColor( );
			Console.Error.WriteLine( );
			Console.Error.Write( "Where:    " );
			Console.ForegroundColor = ConsoleColor.White;
			Console.Error.Write( "\"title\"" );
			Console.ResetColor( );
			Console.Error.WriteLine( "     is the optional caption in the title bar" );
			Console.ForegroundColor = ConsoleColor.White;
			Console.Error.Write( "          \"selected\"" );
			Console.ResetColor( );
			Console.Error.WriteLine( "  is the optional selected printer, either its full name or" );
			Console.Error.WriteLine( "                      a regular expression (default: the default printer name)" );
			Console.ForegroundColor = ConsoleColor.White;
			Console.Error.Write( "          width" );
			Console.ResetColor( );
			Console.Error.WriteLine( "       is the optional width of the input box" );
			Console.Error.WriteLine( "                      (default: 200; minimum: 200; maximum: screen width)" );
			Console.Error.WriteLine( );
			Console.Error.WriteLine( "Notes:    The name of the selected printer is written to Standard Out if \"OK\"" );
			Console.Error.WriteLine( "          is clicked, otherwise an empty string is returned." );
			Console.Error.WriteLine( "          If \"selected\" is specified, the program will try an exact (literal)" );
			Console.Error.WriteLine( "          match first; if no match is found, \"selected\" will be interpreted as" );
			Console.Error.WriteLine( "          a regular expression, and the first match in the sorted printer list" );
			Console.Error.WriteLine( "          will be used." );
			Console.Error.WriteLine( "          Return code 0 for \"OK\", 1 for (command line) errors, 2 for \"Cancel\"." );
			Console.Error.WriteLine( );
			Console.Error.WriteLine( "Credits:  Based on code by Gorkem Gencay on StackOverflow:" );
			Console.ForegroundColor = ConsoleColor.DarkGray;
			Console.Error.WriteLine( "          http://stackoverflow.com/questions/97097" );
			Console.Error.WriteLine( "          /what-is-the-c-sharp-version-of-vb-nets-inputdialog#17546909" );
			Console.ResetColor( );
			Console.Error.WriteLine( );
			Console.Error.WriteLine( "Written by Rob van der Woude" );
			Console.Error.WriteLine( "http://www.robvanderwoude.com" );
			return 1;
		}

		#endregion Error handling
	}

	public enum WMIPrinterStatus
	{
		Other = 1,
		Unknown = 2,
		Idle = 3,
		Printing = 4,
		Warmup = 5,
		StoppedPrinting = 6,
		Offline = 7,
		Paused = 8,
		Error = 9,
		Busy = 10,
		NotAvailable = 11,
		Waiting = 12,
		Processing = 13,
		Initialization = 14,
		PowerSave = 15,
		PendingDeletion = 16,
		IOActive = 17,
		ManualFeed = 18
	}
}
